@@ -30,6 +30,8 @@
 
 ;;; Code:
 
+(require 'compile)
+
 ;;;; Customization
 
 (defgroup rec-mode nil
@@ -37,9 +39,9 @@
   :group 'applications
   :link '(url-link "http://www.gnu.org/software/rec"))
 
-(defcustom rec-open-mode 'edit
+(defcustom rec-open-mode 'navigation
   "Default mode to use when switching a buffer to rec-mode.
-Valid values are `edit' and `navigation'.  The default is `edit'"
+Valid values are `edit' and `navigation'.  The default is `navigation'"
   :type 'symbol
   :group 'rec-mode)
 
@@ -49,6 +51,9 @@ Valid values are `edit' and `navigation'.  The default is `edit'"
 (defvar rec-recinf "recinf"
   "Name of the 'recinf' utility from the GNU recutils.")
 
+(defvar rec-recfix "recfix"
+  "Name of the 'recfix' utility from the GNU recutils.")
+
 ;;;; Variables and constants that the user does not want to touch (really!)
 
 (defconst rec-keyword-rec "%rec"
@@ -96,7 +101,7 @@ Valid values are `edit' and `navigation'.  The default is `edit'"
   "Syntax table used in rec-mode")
 
 (defvar rec-font-lock-keywords
-  `(("^%\\(rec\\|key\\|unique\\|type\\|prohibit\\|mandatory\\|doc\\|fsort\\):" . font-lock-keyword-face)
+  `(("^%\\(rec\\|key\\|unique\\|type\\|prohibit\\|mandatory\\|type\\|doc\\|fsort\\):" . font-lock-keyword-face)
     (,rec-field-name-re . font-lock-variable-name-face)
     ("^\\+" . font-lock-constant-face))
   "Font lock keywords used in rec-mode")
@@ -111,6 +116,8 @@ Valid values are `edit' and `navigation'.  The default is `edit'"
     (define-key map "\C-cl" 'rec-cmd-sel)
     (define-key map "\C-cs" 'rec-cmd-search)
     (define-key map "\C-cm" 'rec-cmd-trim-field-value)
+    (define-key map "\C-cc" 'rec-cmd-compile)
+    (define-key map "\C-cI" 'rec-cmd-show-info)
     (define-key map [remap move-beginning-of-line] 'rec-cmd-beginning-of-line)
     (define-key map (kbd "TAB") 'rec-cmd-goto-next-field)
     (define-key map (concat "\C-c" (kbd "RET")) 'rec-cmd-jump)
@@ -128,15 +135,18 @@ Valid values are `edit' and `navigation'.  The default is `edit'"
     (define-key map "T" 'rec-edit-type)
     (define-key map "B" 'rec-edit-buffer)
     (define-key map "A" 'rec-cmd-append-field)
+    (define-key map "I" 'rec-cmd-show-info)
     (define-key map "t" 'rec-cmd-show-descriptor)
     (define-key map "l" 'rec-cmd-sel)
     (define-key map "s" 'rec-cmd-search)
     (define-key map "m" 'rec-cmd-trim-field-value)
+    (define-key map "c" 'rec-cmd-compile)
     (define-key map "\C-ct" 'rec-find-type)
     (define-key map [remap move-beginning-of-line] 'rec-cmd-beginning-of-line)
     (define-key map "#" 'rec-cmd-count)
     (define-key map (kbd "RET") 'rec-cmd-jump)
     (define-key map (kbd "TAB") 'rec-cmd-goto-next-field)
+    (define-key map (kbd "SPC") 'rec-cmd-toggle-field-visibility)
     (define-key map "b" 'rec-cmd-jump-back)
     map)
   "Keymap for rec-mode")
@@ -154,14 +164,21 @@ Return a list whose first element is the symbol 'comment and the
 second element is the string with the contents of the comment,
 including the leading #:
 
-   (comment \"# foo\")
+   (comment POSITION \"# foo\")
 
-If the point is not at the beginning of a comment return nil"
-  (when (and (equal (current-column) 0)
-             (looking-at rec-comment-re))
-    (goto-char (match-end 0))
-    (list 'comment (buffer-substring-no-properties (match-beginning 0)
-                                                   (match-end 0)))))
+If the point is not at the beginning of a comment then return nil"
+  (let ((there (point))
+        comment)
+    (when (and (equal (current-column) 0)
+               (looking-at rec-comment-re))
+      (setq comment (list 'comment
+                          there
+                          (buffer-substring-no-properties (match-beginning 0)
+                                                          (match-end 0))))
+      (goto-char (match-end 0))
+      ;; Skip a newline if needed
+      (when (looking-at "\n") (goto-char (match-end 0))))
+    comment))
 
 (defun rec-parse-field-name ()
   "Parse and return a field name starting at point.
@@ -169,7 +186,7 @@ If the point is not at the beginning of a comment return nil"
 Return a list with whose elements are the parts of the field
 name.  For the name a:b:c:d: the following list is returned:
 
-   ('a' 'b' 'c' 'd')
+   (\"a\" \"b\" \"c\" \"d\")
 
 If the point is not at the beginning of a field name return nil"
   (when (and (equal (current-column) 0)
@@ -218,14 +235,6 @@ nil"
               (delete-char 1)))
         (setq val (buffer-substring-no-properties (point-min)
                                                   (point-max))))
-;;      (with-temp-buffer
-;;        (insert val)
-;;        (goto-char (point-min))
-;;        (when (looking-at "[ \t\n]+")
-;;          (delete-region (match-beginning 0)
-;;                         (match-end 0)))
-;;        (setq val (buffer-substring-no-properties (point-min)
-;;                                                  (point-max))))
       val)))
 
 (defun rec-parse-field ()
@@ -236,7 +245,7 @@ The returned structure is a list whose first element is the
 symbol 'field', the second element is the name of the field and
 the second element is the value of the field:
 
-   (field FIELD-NAME FIELD-VALUE)
+   (field POSITION FIELD-NAME FIELD-VALUE)
 
 If the pointer is not at the beginning of a field
 descriptor then return nil"
@@ -245,6 +254,8 @@ descriptor then return nil"
     (and (setq field-name (rec-parse-field-name))
          (setq field-value (rec-parse-field-value)))
     (when (and field-name field-value)
+        ;; Skip a newline if needed
+        (when (looking-at "\n") (goto-char (match-end 0)))
         (list 'field there field-name field-value))))
 
 (defun rec-parse-record ()
@@ -261,9 +272,7 @@ nil"
         record field-or-comment)
     (while (setq field-or-comment (or (rec-parse-field)
                                       (rec-parse-comment)))
-      (setq record (cons field-or-comment record))
-      ;; Skip the newline finishing the field or the comment
-      (when (looking-at "\n") (goto-char (match-end 0))))
+      (setq record (cons field-or-comment record)))
     (setq record (list 'record there (reverse record)))))
 
 ;;;; Writer functions (rec-insert-*)
@@ -274,9 +283,8 @@ nil"
 
 (defun rec-insert-comment (comment)
   "Insert the written form of COMMENT in the current buffer"
-  (when (and (listp comment)
-             (equal (car comment) 'comment))
-    (insert (cadr comment))))
+  (when (rec-comment-p comment)
+    (insert (rec-comment-string comment))))
 
 (defun rec-insert-field-name (field-name)
   "Insert the written form of FIELD-NAME in the current buffer"
@@ -295,36 +303,19 @@ nil"
 
 (defun rec-insert-field (field)
   "Insert the written form of FIELD in the current buffer"
-  (when (and (listp field)
-             (equal (car field) 'field))
-    (when (rec-insert-field-name (cadr field))
+  (when (rec-field-p field)
+    (when (rec-insert-field-name (rec-field-name field))
       (insert " ")
-      (rec-insert-field-value (nth 2 field)))))
+      (rec-insert-field-value (rec-field-value field)))))
 
-(defun rec-insert-record (record &optional fields)
-  "Insert the written form of RECORD in the current buffer.
-
-If FIELDS is specified, it is a list of fields to include in the
-insertion.  If a field in the list does not exist in the record
-it is ignored."
-  (when (and (listp record)
-             (equal (car record) 'record))
-    (rec-insert-record-2 (cdr record) fields)))
-
-(defun rec-insert-record-2 (record fields)
-  "Insert the written form of RECORD in the current buffer.
-Recursive part"
-  (when (and record (listp record))
-    (let ((elem (car record)))
-      (cond
-       ((and (not fields)
-             (equal (car elem) 'comment))
-        (rec-insert-comment elem))
-       ((and (equal (car elem) 'field)
-             (or (not fields)
-                 (member (nth 1 elem) fields)))
-        (rec-insert-field elem))))
-    (rec-insert-record-2 (cdr record) fields)))
+(defun rec-insert-record (record)
+  "Insert the written form of RECORD in the current buffer."
+  (when (rec-record-p record)
+    (mapcar (lambda (elem)
+              (cond
+               ((rec-comment-p elem) (rec-insert-comment elem))
+               ((rec-field-p elem) (rec-insert-field elem))))
+            (rec-record-fields record))))
 
 ;;;; Operations on record structures
 ;;
@@ -346,35 +337,57 @@ Recursive part"
   (when (rec-record-p record)
     (nth 2 record)))
 
+(defun rec-record-descriptor-p (record)
+  "Determine if the given record is a descriptor."
+  (not (null (rec-record-assoc '("%rec") record))))
+
 (defun rec-record-assoc (name record)
   "Get a list with the values of the fields in RECORD named NAME.
 
-XXX: describe NAME.
+NAME shall be a field name i.e. a list of field name parts.
 
 If no such field exists in RECORD then nil is returned."
   (if (stringp name)
       (setq name (rec-parse-field-name-from-string name)))
-  (when (and (listp record)
-             (equal (car record) 'record))
+  (when (rec-record-p record)
     (let (result)
       (mapcar (lambda (field)
-                (when (and (equal (car field) 'field)
+                (when (and (rec-field-p field)
                            (equal name (rec-field-name field)))
-                  (setq result (cons (nth 3 field) result))))
+                  (setq result (cons (rec-field-value field) result))))
               (rec-record-fields record))
       (reverse result))))
 
 (defun rec-record-names (record)
   "Get a list of the field names in the record"
-  (when (and (listp record)
-             (equal (car record) 'record))
+  (when (rec-record-p record)
     (let (result)
       (mapcar (lambda (field)
-                (when (and (equal (car field) 'field))
-                  (setq result (cons (nth 1 field) result))))
-              (cdr record))
+                (when (rec-field-p field)
+                  (setq result (cons (rec-field-name field) result))))
+              (rec-record-fields record))
       (reverse result))))
 
+;;;; Operations on comment structures
+;;
+;; Those functions retrieve or set properties of comment structures.
+
+(defun rec-comment-p (comment)
+  "Determine if the provided structure is a comment"
+  (and (listp comment)
+       (= (length comment) 3)
+       (equal (car comment) 'comment)))
+
+(defun rec-comment-position (comment)
+  "Return the start position of the given comment."
+  (when (rec-comment-p comment)
+    (nth 1 comment)))
+
+(defun rec-comment-string (comment)
+  "Return the string composig the comment, including the initial '#' character."
+  (when (rec-comment-p comment)
+    (nth 2 comment)))
+
 ;;;; Operations on field structures
 ;;
 ;; Those functions retrieve or set properties of field structures.
@@ -400,10 +413,17 @@ If no such field exists in RECORD then nil is returned."
   (when (rec-field-p field)
     (nth 3 field)))
 
+(defun rec-field-set-value (field value)
+  "Return FIELD with its value replaced by VALUE."
+  (list 'field
+        (rec-field-position field)
+        (rec-field-name field)
+        value))
+
 (defun rec-field-trim-value (field)
-  "Trim the value of the given field."
+  "Return FIELD with its value trimmed."
   (when (rec-field-p field)
-    (let ((value (nth 2 field))
+    (let ((value (rec-field-value field))
           c)
       (with-temp-buffer
         (insert value)
@@ -422,7 +442,7 @@ If no such field exists in RECORD then nil is returned."
         (delete-region (point) (point-max))
         (setq value (buffer-substring-no-properties (point-min)
                                                     (point-max))))
-      (setcar (cddr field) value))))
+      (rec-field-set-value field value))))
 
 ;;;; Get entities under pointer
 ;;
@@ -629,6 +649,9 @@ file."
 (defun rec-goto-type (type)
   "Goto the beginning of the descriptor with type TYPE.
 
+If there are records of type TYPE in the record set then goto the
+first record.  Otherwise goto to the record descriptor.
+
 If the type do not exist in the current buffer then
 this function returns nil."
   (if (or (not type)
@@ -946,7 +969,14 @@ the result buffer."
   "Show the records of the given type"
   (widen)
   (unless (rec-goto-type type)
-    (message "No records with that type found in the file"))
+    (message "No records of the requested type were found."))
+  ;; Show the first data record of this type, if it exists.
+  (if (and (not type)
+           (save-excursion
+             (let ((record-type (rec-record-type)))
+               (and (rec-goto-next-rec)
+                    (equal (rec-record-type) record-type)))))
+      (rec-goto-next-rec))
   (rec-show-record))
 
 (defun rec-show-record ()
@@ -956,15 +986,21 @@ the result buffer."
   (use-local-map rec-mode-map)
   ;; TODO: Update field names for autocompletion
   ;;  (let ((names (rec-record-field-names (rec-current-record)))))
+  (rec-set-head-line nil)
   (rec-set-mode-line (rec-record-type)))
 
-;;;; Mode line
+;;;; Mode line and Head line
 
 (defun rec-set-mode-line (str)
-  "Set the modeline in rec buffers"
-  (setq mode-line-buffer-identification
-        (list 20
-              "%b " str)))
+  "Set the modeline in rec buffers."
+  (when str
+    (setq mode-line-buffer-identification
+          (list 20
+                "%b " str))))
+
+(defun rec-set-head-line (str)
+  "Set the headline in rec buffers."
+  (setq header-line-format str))
 
 ;;;; Fast selection
 
@@ -1227,7 +1263,7 @@ buffer"
           (insert field-value)
           (switch-to-buffer-other-window edit-buf)
           (goto-char (point-min))
-          (message "Edit the value of the field and press C-c C-c to exit"))
+          (message "Edit the value of the field and use C-c C-c to exit"))
       (message "Not in a field"))))
 
 (defun rec-finish-editing-field ()
@@ -1335,10 +1371,16 @@ record.  Interactive version."
 file.  Interactive version."
   (interactive)
   (widen)
-  (if (save-excursion
-        (not (rec-goto-next-rec)))
-      (message "No more records")
-    (rec-goto-next-rec))
+  (let ((record-type (rec-record-type)))
+    (if (save-excursion
+          (and (rec-goto-next-rec)
+               (equal (rec-record-type) record-type)
+               (not (rec-record-descriptor-p (rec-current-record)))))
+        (rec-goto-next-rec)
+      (if (not (rec-record-type))
+          (message "No more records")
+        (message (concat "No more records of type "
+                         (rec-record-type))))))
   (unless rec-editing
     (rec-show-record)))
 
@@ -1347,10 +1389,16 @@ file.  Interactive version."
 the file.  Interactive version."
   (interactive)
   (widen)
-  (if (save-excursion
-        (not (rec-goto-previous-rec)))
-      (message "No more records")
-    (rec-goto-previous-rec))
+  (let ((record-type (rec-record-type)))
+    (if (save-excursion
+          (and (rec-goto-previous-rec)
+               (equal (rec-record-type) record-type)
+               (not (rec-record-descriptor-p (rec-current-record)))))
+        (rec-goto-previous-rec)
+      (if (not (rec-record-type))
+          (message "No more records")
+        (message (concat "No more records of type "
+                         (rec-record-type))))))
   (unless rec-editing
     (rec-show-record)))
 
@@ -1407,10 +1455,10 @@ point."
   (setq rec-editing t)
   (setq buffer-read-only nil)
   (use-local-map rec-mode-edit-map)
+  (rec-set-head-line "Editing record - use C-cC-c to return to navigation mode")
   (rec-set-mode-line "Edit record")
   (setq rec-update-p t)
-  (setq rec-preserve-last-newline t)
-  (message "Editing: Press C-c C-c when you are done"))
+  (setq rec-preserve-last-newline t))
 
 (defun rec-edit-type ()
   "Go to the type edition mode"
@@ -1422,8 +1470,10 @@ point."
   (rec-narrow-to-type (rec-record-type))
   (setq rec-update-p t)
   (goto-char (point-min))
-  (rec-set-mode-line "Edit type")
-  (message "Editing:  Press C-c C-c when you are done"))
+  (rec-set-head-line (concat "Editing type "
+                             "'" (rec-record-type) "'"
+                             " - use C-cC-c to return to navigation mode"))
+  (rec-set-mode-line "Edit type"))
 
 (defun rec-edit-buffer ()
   "Go to the buffer edition mode"
@@ -1434,8 +1484,8 @@ point."
   (widen)
   (setq rec-update-p t)
   (goto-char (point-min))
-  (rec-set-mode-line "Edit buffer")
-  (message "Editing: Press C-c C-c when you are done"))
+  (rec-set-head-line "Editing buffer - use C-cC-c to return to navigation mode")
+  (rec-set-mode-line "Edit buffer"))
 
 (defun rec-finish-editing ()
   "Go back from the record edition mode"
@@ -1454,7 +1504,7 @@ point."
       (rec-update-buffer-descriptors))
     (setq rec-update-p nil))
   (rec-show-record)
-  ;; TODO: Restore modeline
+  (rec-set-head-line nil)
   (rec-set-mode-line (rec-record-type))
   (setq rec-editing nil)
   (rec-init-selections)
@@ -1530,10 +1580,50 @@ records of the current type"
   (save-excursion
     (let ((buffer-read-only nil)
           (field (rec-current-field)))
-      (rec-field-trim-value field)
+      (setq field (rec-field-trim-value field))
       (rec-delete-field)
       (rec-insert-field field))))
 
+(defun rec-cmd-compile ()
+  "Compile the current file with recfix."
+  (interactive)
+  (let ((cur-buf (current-buffer))
+        (cmd (concat rec-recfix " "))
+        (tmpfile (make-temp-file "rec-mode-")))
+    (if buffer-file-name
+        (setq cmd (concat cmd buffer-file-name))
+      (with-temp-file tmpfile
+        (insert-buffer cur-buf))
+      (setq cmd (concat cmd tmpfile)))
+    (compilation-start cmd)))
+
+(defun rec-cmd-show-info ()
+  "Show information about the recfile in the modeline."
+  (interactive)
+  (let ((cur-buf (current-buffer))
+        (filename (if buffer-file-name
+                      buffer-file-name
+                    (make-temp-file "rec-mode-")))
+        (msg ""))
+    (if (not buffer-file-name)
+        (with-temp-file tmpfile
+          (insert-buffer cur-buf)))
+    (with-temp-buffer
+      (call-process rec-recinf
+                    nil ; infile
+                    t   ; output to current buffer
+                    nil ; display
+                    filename)
+      (setq msg (buffer-substring-no-properties (point-min)
+                                                (point-max))))
+    ;; Delete temporary file.
+    (if (not buffer-file-name)
+        (delete-file filename))
+    ;; Show the message.
+    (setq msg (replace-regexp-in-string "\n$" "" msg))
+    (setq msg (replace-regexp-in-string "\n" ", " msg))
+    (message msg)))
+
 (defun rec-cmd-beginning-of-line ()
   "Move the point to the beginning of the current line.
 
@@ -1583,7 +1673,7 @@ Commands:
   ;; Goto the first record of the first type (including the Unknown)
   (rec-update-buffer-descriptors)
   ;; Initialize the value of rec-custom-searches
-  (rec-init-selections)
+;;  (rec-init-selections)
   (if (equal rec-open-mode 'navigation)
     (progn
       (setq buffer-read-only t)
