@@ -1,4 +1,4 @@
-/* -*- mode: C -*- Time-stamp: "2010-11-27 10:24:19 jemarch"
+/* -*- mode: C -*- Time-stamp: "2010-12-19 17:46:14 jemarch"
  *
  *       File:         rec-int.c
  *       Date:         Thu Jul 15 18:23:26 2010
@@ -29,7 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <regex.h>
-#include <libintl.h>
+#include <gettext.h>
 #define _(str) dgettext (PACKAGE, str)
 #include <tempname.h>
 
@@ -44,20 +44,20 @@
  * Forward references.
  */
 
-static int rec_int_check_descriptor (rec_rset_t rset, FILE *errors);
+static int rec_int_check_descriptor (rec_rset_t rset, rec_buf_t errors);
 static int rec_int_check_record_key (rec_rset_t rset,
                                      rec_record_t orig_record, rec_record_t record,
-                                     FILE *errors);
+                                     rec_buf_t errors);
 static int rec_int_check_record_types (rec_db_t db,
                                        rec_rset_t rset,
                                        rec_record_t record,
-                                       FILE *errors);
+                                       rec_buf_t errors);
 static int rec_int_check_record_mandatory (rec_rset_t rset, rec_record_t record,
-                                           FILE *errors);
+                                           rec_buf_t errors);
 static int rec_int_check_record_unique (rec_rset_t rset, rec_record_t record,
-                                        FILE *errors);
+                                        rec_buf_t errors);
 static int rec_int_check_record_prohibit (rec_rset_t rset, rec_record_t record,
-                                          FILE *errors);
+                                          rec_buf_t errors);
 static void rec_int_merge_remote (rec_rset_t rset);
 static bool rec_int_rec_type_p (char *str);
 
@@ -69,27 +69,24 @@ int
 rec_int_check_db (rec_db_t db,
                   bool check_descriptors_p,
                   bool remote_descriptors_p,
-                  FILE *errors)
+                  rec_buf_t errors)
 {
-  bool ret;
+  int ret;
   size_t db_size;
   size_t n_rset;
   rec_rset_t rset;
   
-  ret = true;
+  ret = 0;
 
   db_size = rec_db_size (db);
   for (n_rset = 0; n_rset < db_size; n_rset++)
     {
       rset = rec_db_get_rset (db, n_rset);
-      if (rec_int_check_rset (db,
-                              rset,
-                              check_descriptors_p,
-                              remote_descriptors_p,
-                              errors) > 0)
-        {
-          ret = false;
-        }
+      ret = ret + rec_int_check_rset (db,
+                                      rset,
+                                      check_descriptors_p,
+                                      remote_descriptors_p,
+                                      errors);
     }
 
   return ret;
@@ -100,7 +97,7 @@ rec_int_check_rset (rec_db_t db,
                     rec_rset_t rset,
                     bool check_descriptor_p,
                     bool remote_descriptor_p,
-                    FILE *errors)
+                    rec_buf_t errors)
 {
   int res;
   rec_rset_elem_t rset_elem;
@@ -146,7 +143,7 @@ rec_int_check_record (rec_db_t db,
                       rec_rset_t rset,
                       rec_record_t orig_record,
                       rec_record_t record,
-                      FILE *errors)
+                      rec_buf_t errors)
 {
   int res;
 
@@ -164,7 +161,7 @@ bool
 rec_int_check_field_type (rec_db_t db,
                           rec_rset_t rset,
                           rec_field_t field,
-                          FILE *errors)
+                          rec_buf_t errors)
 {
   bool res;
   rec_type_reg_t type_reg;
@@ -175,6 +172,7 @@ rec_int_check_field_type (rec_db_t db,
   rec_type_t referring_type;
   rec_type_t referred_type;
   char *errors_str;
+  char *tmp;
 
   res = true;
   rset_name = NULL;
@@ -235,11 +233,13 @@ rec_int_check_field_type (rec_db_t db,
           && errors)
         {
           /* Emit a warning.  */
-          fprintf (errors, _("%s:%s: warning: type %s collides with referred type %s in the rset %s.\n"),
-                   rec_field_source (field), rec_field_location_str (field),
-                   rec_type_kind_str (referred_type),
-                   rec_type_kind_str (referring_type),
-                   rset_name);
+          asprintf (&tmp, _("%s:%s: warning: type %s collides with referred type %s in the rset %s.\n"),
+                     rec_field_source (field), rec_field_location_str (field),
+                     rec_type_kind_str (referred_type),
+                     rec_type_kind_str (referring_type),
+                     rset_name);
+          rec_buf_puts (tmp, errors);
+          free (tmp);
         }
 
       type = referring_type;
@@ -255,9 +255,14 @@ rec_int_check_field_type (rec_db_t db,
     {
       if (!rec_type_check (type, rec_field_value (field), &errors_str))
         {
-          fprintf (errors, "%s:%s: error: %s\n",
-                   rec_field_source (field), rec_field_location_str (field),
-                   errors_str);
+          if (errors)
+            {
+              asprintf (&tmp, "%s:%s: error: %s\n",
+                         rec_field_source (field), rec_field_location_str (field),
+                         errors_str);
+              rec_buf_puts (tmp, errors);
+              free (tmp);
+            }
           free (errors_str);
           res = false;
         }
@@ -270,7 +275,7 @@ static int
 rec_int_check_record_types (rec_db_t db,
                             rec_rset_t rset,
                             rec_record_t record,
-                            FILE *errors)
+                            rec_buf_t errors)
 {
   int res;
   rec_record_elem_t rec_elem;
@@ -296,7 +301,7 @@ rec_int_check_record_types (rec_db_t db,
 static int
 rec_int_check_record_mandatory (rec_rset_t rset,
                                 rec_record_t record,
-                                FILE *errors)
+                                rec_buf_t errors)
 {
   int res;
   rec_record_t descriptor;
@@ -306,6 +311,7 @@ rec_int_check_record_mandatory (rec_rset_t rset,
   char *mandatory_field_str;
   rec_field_t field;
   size_t i, j, num_fields;
+  char *tmp;
   
   res = 0;
 
@@ -334,11 +340,13 @@ rec_int_check_record_mandatory (rec_rset_t rset,
               if (rec_record_get_num_fields_by_name (record, mandatory_field_name)
                   == 0)
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: mandatory field '%s' not found in record\n"),
-                           rec_record_source (record),
-                           rec_record_location_str (record),
-                           mandatory_field_str);
+                  asprintf (&tmp,
+                             _("%s:%s: error: mandatory field '%s' not found in record\n"),
+                             rec_record_source (record),
+                             rec_record_location_str (record),
+                             mandatory_field_str);
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
             }
@@ -353,7 +361,7 @@ rec_int_check_record_mandatory (rec_rset_t rset,
 static int
 rec_int_check_record_unique (rec_rset_t rset,
                              rec_record_t record,
-                             FILE *errors)
+                             rec_buf_t errors)
 {
   int res;
   rec_record_t descriptor;
@@ -363,6 +371,7 @@ rec_int_check_record_unique (rec_rset_t rset,
   char *unique_field_str;
   rec_field_t field;
   size_t i, j, num_fields;
+  char *tmp;
   
   res = 0;
 
@@ -391,11 +400,13 @@ rec_int_check_record_unique (rec_rset_t rset,
               if (rec_record_get_num_fields_by_name (record, unique_field_name)
                   > 1)
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: field '%s' shall be unique in this record\n"),
-                           rec_record_source (record),
-                           rec_record_location_str (record),
-                           unique_field_str);
+                  asprintf (&tmp,
+                             _("%s:%s: error: field '%s' shall be unique in this record\n"),
+                             rec_record_source (record),
+                             rec_record_location_str (record),
+                             unique_field_str);
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
             }
@@ -410,7 +421,7 @@ rec_int_check_record_unique (rec_rset_t rset,
 static int
 rec_int_check_record_prohibit (rec_rset_t rset,
                                rec_record_t record,
-                               FILE *errors)
+                               rec_buf_t errors)
 {
   int res;
   rec_record_t descriptor;
@@ -420,6 +431,7 @@ rec_int_check_record_prohibit (rec_rset_t rset,
   char *prohibit_field_str;
   rec_field_t field;
   size_t i, j, num_fields;
+  char *tmp;
   
   res = 0;
 
@@ -448,11 +460,13 @@ rec_int_check_record_prohibit (rec_rset_t rset,
               if (rec_record_get_num_fields_by_name (record, prohibit_field_name)
                   > 0)
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: prohibited field '%s' found in record\n"),
-                           rec_record_source (record),
-                           rec_record_location_str (record),
-                           prohibit_field_str);
+                  asprintf (&tmp,
+                             _("%s:%s: error: prohibited field '%s' found in record\n"),
+                             rec_record_source (record),
+                             rec_record_location_str (record),
+                             prohibit_field_str);
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
             }
@@ -468,7 +482,7 @@ static int
 rec_int_check_record_key (rec_rset_t rset,
                           rec_record_t orig_record,
                           rec_record_t record,
-                          FILE *errors)
+                          rec_buf_t errors)
 {
   int res;
   rec_record_t descriptor;
@@ -482,6 +496,7 @@ rec_int_check_record_key (rec_rset_t rset,
   bool duplicated_key;
   size_t i;
   size_t num_fields;
+  char *tmp;
   
   res = 0;
 
@@ -503,20 +518,24 @@ rec_int_check_record_key (rec_rset_t rset,
 
               if (num_fields == 0)
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: key field '%s' not found in record\n"),
-                           rec_record_source (record),
-                           rec_record_location_str (record),
-                           rec_field_value (field));
+                  asprintf (&tmp,
+                             _("%s:%s: error: key field '%s' not found in record\n"),
+                             rec_record_source (record),
+                             rec_record_location_str (record),
+                             rec_field_value (field));
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
               else if (num_fields > 1)
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: multiple key fields '%s' in record\n"),
-                           rec_record_source (record),
-                           rec_record_location_str (record),
-                           rec_field_value (field));
+                  asprintf (&tmp,
+                             _("%s:%s: error: multiple key fields '%s' in record\n"),
+                             rec_record_source (record),
+                             rec_record_location_str (record),
+                             rec_field_value (field));
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
               else  /* num_fields == 1 */
@@ -555,11 +574,13 @@ rec_int_check_record_key (rec_rset_t rset,
 
                   if (duplicated_key)
                     {
-                      fprintf (errors,
-                               _("%s:%s: error: duplicated key value in field '%s' in record\n"),
-                               rec_record_source (orig_record),
-                               rec_record_location_str (orig_record),
-                               rec_field_name_str (key));
+                      asprintf (&tmp,
+                                 _("%s:%s: error: duplicated key value in field '%s' in record\n"),
+                                 rec_record_source (orig_record),
+                                 rec_record_location_str (orig_record),
+                                 rec_field_name_str (key));
+                      rec_buf_puts (tmp, errors);
+                      free (tmp);
                       res++;
                       break;
                     }
@@ -577,7 +598,7 @@ rec_int_check_record_key (rec_rset_t rset,
 
 static int
 rec_int_check_descriptor (rec_rset_t rset,
-                          FILE *errors)
+                          rec_buf_t errors)
 {
   int res;
   rec_record_t descriptor;
@@ -592,6 +613,7 @@ rec_int_check_descriptor (rec_rset_t rset,
   rec_field_name_t prohibit_fname;
   char *field_value;
   rec_fex_t fex;
+  char *tmp;
 
   res = 0;
   descriptor = rec_rset_descriptor (rset);
@@ -605,33 +627,55 @@ rec_int_check_descriptor (rec_rset_t rset,
       unique_fname = rec_parse_field_name_str ("%unique:");
       prohibit_fname = rec_parse_field_name_str ("%prohibit:");
 
-      /* Check the type of the record set.  */
-      field = rec_record_get_field_by_name (descriptor, rec_fname, 0);
-      if (!field)
+      /* Check the type of the record set:
+
+         1. There should be one (and only one) %rec: field in the
+            record.
+         2. The value of the %rec: field shall be well-formed.
+      */
+      if (rec_record_get_num_fields_by_name (descriptor, rec_fname) == 0)
         {
-          fprintf (errors,
-                   _("%s:%s: error: missing %%rec field in record descriptor\n"),
-                   rec_record_source (descriptor),
-                   rec_record_location_str (descriptor));
+          asprintf (&tmp,
+                     _("%s:%s: error: missing %%rec field in record descriptor\n"),
+                     rec_record_source (descriptor),
+                     rec_record_location_str (descriptor));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
           res++;
         }
+      else if (rec_record_get_num_fields_by_name (descriptor, rec_fname) > 1)
+        {
+          asprintf (&tmp,
+                     _("%s:%s: error: too many %%rec fields in record descriptor\n"),
+                     rec_record_source (descriptor),
+                     rec_record_location_str (descriptor));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
+          res++;
+        }
+
+      field = rec_record_get_field_by_name (descriptor, rec_fname, 0);
       if (!rec_int_rec_type_p (rec_field_value (field)))
         {
-          fprintf (errors,
-                   _("%s:%s: error: invalid record type %s\n"),
-                   rec_field_source (field),
-                   rec_field_location_str (field),
-                   rec_field_value (field));
+          asprintf (&tmp,
+                     _("%s:%s: error: invalid record type %s\n"),
+                     rec_field_source (field),
+                     rec_field_location_str (field),
+                     rec_field_value (field));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
           res++;
         }
 
       /* Only one 'key:' entry is allowed, if any.  */
       if (rec_record_get_num_fields_by_name (descriptor, key_fname) > 1)
         {
-          fprintf (errors,
-                   _("%s:%s: error: only one %%key field is allowed in a record descriptor\n"),
-                   rec_record_source (descriptor),
-                   rec_record_location_str (descriptor));
+          asprintf (&tmp,
+                     _("%s:%s: error: only one %%key field is allowed in a record descriptor\n"),
+                     rec_record_source (descriptor),
+                     rec_record_location_str (descriptor));
+          rec_buf_puts (tmp, errors);
+          free (tmp);
           res++;
         }
 
@@ -649,10 +693,12 @@ rec_int_check_descriptor (rec_rset_t rset,
               if (!rec_type_descr_p (field_value))
                 {
                   /* XXX: make rec_type_descr_p to report more details.  */
-                  fprintf (errors,
-                           _("%s:%s: error: invalid type specification\n"),
-                           rec_field_source (field),
-                           rec_field_location_str (field));
+                  asprintf (&tmp,
+                             _("%s:%s: error: invalid type specification\n"),
+                             rec_field_source (field),
+                             rec_field_location_str (field));
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
             }
@@ -669,12 +715,14 @@ rec_int_check_descriptor (rec_rset_t rset,
                 }
               else
                 {
-                  fprintf (errors,
-                           _("%s:%s: error: value for %s[%d] is not a list of field names\n"),
-                           rec_record_source (descriptor),
-                           rec_record_location_str (descriptor),
-                           rec_field_name_str (field),
-                           rec_record_get_field_index_by_name (descriptor, field));
+                  asprintf (&tmp,
+                             _("%s:%s: error: value for %s[%d] is not a list of field names\n"),
+                             rec_record_source (descriptor),
+                             rec_record_location_str (descriptor),
+                             rec_field_name_str (field),
+                             rec_record_get_field_index_by_name (descriptor, field));
+                  rec_buf_puts (tmp, errors);
+                  free (tmp);
                   res++;
                 }
             }
@@ -708,8 +756,8 @@ rec_int_merge_remote (rec_rset_t rset)
   char *rec_url = NULL;
   char *rec_file = NULL;
   char *rec_source = NULL;
-  int tmpfile_des;
   FILE *external_file;
+  int tmpfile_des;
   char tmpfile_name[14];
 
   rec_fname = rec_parse_field_name_str ("%rec:");
