@@ -25,14 +25,15 @@
 
 #include <config.h>
 
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include <libintl.h>
+#include <gettext.h>
 #define _(str) dgettext (PACKAGE, str)
 #define N_(str) gettext_noop (str)
 
 #include <rec.h>
+#include <rec-utils.h>
 
 /*
  * Static functions defined in this file
@@ -49,30 +50,10 @@ static bool rec_parse_comment (rec_parser_t parser, rec_comment_t *comment);
 
 static bool rec_parser_digit_p (char c);
 static bool rec_parser_letter_p (char c);
+static bool rec_parser_init_common (rec_parser_t parser, char *source);
 
-/* Buffer management */
-
-#define REC_PARSER_BUF_STEP 512
-
-struct rec_parser_buf_s
-{
-  char *data;
-  size_t used;
-  size_t size;
-};
-
-typedef struct rec_parser_buf_s *rec_parser_buf_t;
-
-static rec_parser_buf_t rec_parser_buf_new ();
-static bool rec_parser_buf_add (rec_parser_buf_t buf,
-                                char c);
-static void rec_parser_buf_rewind (rec_parser_buf_t buf, int n);
-static char *rec_parser_buf_data (rec_parser_buf_t buf);
-static void rec_parser_buf_adjust (rec_parser_buf_t buf);
-static void rec_parser_buf_destroy (rec_parser_buf_t buf);
-
-/* Parser Data Structure
- *
+/*
+ * Parser Data Structure
  */
 
 enum rec_parser_error_e
@@ -90,8 +71,11 @@ enum rec_parser_error_e
 
 struct rec_parser_s
 {
-  FILE *in; /* File stream used by the parser. */
-  char *file_name;
+  FILE *in_file;     /* File stream used by the parser.  */
+  char *in_buffer;   /* Buffer used by the parser.  */
+  char *p;           /* Pointer to the next unreaded character in
+                        in_buffer */
+  char *source;
   
   rec_record_t prev_descriptor;
 
@@ -119,27 +103,43 @@ const char *rec_parser_error_strings[] =
 
 rec_parser_t
 rec_parser_new (FILE *in,
-                char *file_name)
+                char *source)
 {
   rec_parser_t parser;
 
   parser = malloc (sizeof (struct rec_parser_s));
   if (parser != NULL)
     {
-      parser->in = in;
-      if (file_name)
+      parser->in_file = in;
+      parser->in_buffer = NULL;
+
+      if (!rec_parser_init_common (parser, source))
         {
-          parser->file_name = strdup (file_name);
+          free (parser);
+          parser = NULL;
         }
-      else
+    }
+
+  return parser;
+}
+
+rec_parser_t
+rec_parser_new_str (char *buffer,
+                    char *source)
+{
+  rec_parser_t parser;
+
+  parser = malloc (sizeof (struct rec_parser_s));
+  if (parser != NULL)
+    {
+      parser->in_buffer = buffer;
+      parser->in_file = NULL;
+
+      if (!rec_parser_init_common (parser, source))
         {
-          parser->file_name = NULL;
+          free (parser);
+          parser = NULL;
         }
-      parser->eof = false;
-      parser->error = false;
-      parser->line = 1;
-      parser->character = 0;
-      parser->prev_descriptor = NULL;
     }
 
   return parser;
@@ -148,7 +148,7 @@ rec_parser_new (FILE *in,
 void
 rec_parser_destroy (rec_parser_t parser)
 {
-  free (parser->file_name);
+  free (parser->source);
   free (parser);
 }
 
@@ -175,14 +175,9 @@ rec_parser_perror (rec_parser_t parser,
   va_start (ap, fmt);
   vfprintf (stderr, fmt, ap);
   fputs (":", stderr);
-  /* XXX: replace the following line with a calculation of the number
-     of digits of a given number.  Sorry, I am a bit drunk while
-     writing this and cannot think clearly :D -jemarch */
-  number_str = malloc (10); /* 10 is a big arbitrary number */
-
   if (number_str)
     {
-      sprintf(number_str, "%d", parser->line);
+      asprintf (&number_str, "%zu", parser->line);
       fputs (number_str, stderr);
       fputs (": error: ", stderr);
       fputs (gettext (rec_parser_error_strings[parser->error]), stderr);
@@ -288,6 +283,7 @@ rec_parser_reset (rec_parser_t parser)
 {
   parser->eof = false;
   parser->error = REC_PARSER_NOERROR;
+  parser->p = parser->in_buffer;
 }
 
 bool
@@ -325,7 +321,7 @@ rec_parse_field (rec_parser_t parser,
           new = rec_field_new (field_name,
                                field_value);
 
-          rec_field_set_source (new, parser->file_name);
+          rec_field_set_source (new, parser->source);
           rec_field_set_location (new, location);
           rec_field_set_char_location (new, char_location);
           *field = new;
@@ -366,7 +362,7 @@ rec_parse_record (rec_parser_t parser,
     }
 
   /* Localize the potential record.  */
-  rec_record_set_source (new, parser->file_name);
+  rec_record_set_source (new, parser->source);
   rec_record_set_location (new, parser->line);
   char_location = parser->character;
   if (char_location != 0)
@@ -610,7 +606,6 @@ rec_parse_field_name_str (char *str)
 {
   rec_parser_t parser;
   rec_field_name_t field_name;
-  FILE *stm;
   char *str2;
   size_t str_size;
 
@@ -635,32 +630,24 @@ rec_parse_field_name_str (char *str)
     }
   
   field_name = NULL;
-  stm = fmemopen (str2, strlen (str2), "r");
-  if (stm)
+
+  parser = rec_parser_new_str (str2, "dummy");
+  if (!rec_parse_field_name (parser, &field_name))
     {
-      parser = rec_parser_new (stm, "dummy");
-      if (parser)
-        {
-          if (!rec_parse_field_name (parser, &field_name))
-            {
-              field_name = NULL;
-            }
-          rec_parser_destroy (parser);
-        }
-
-      if (!feof (stm))
-        {
-          /* There is additional stuff after the field name.  */
-          if (field_name)
-            {
-              rec_field_name_destroy (field_name);
-            }
-          field_name = NULL;
-        }
-
-      fclose (stm);
+      field_name = NULL;
     }
 
+  if (!rec_parser_eof (parser))
+    {
+      /* There is additional stuff after the field name.  */
+      if (field_name)
+        {
+          rec_field_name_destroy (field_name);
+        }
+      field_name = NULL;
+    }
+
+  rec_parser_destroy (parser);
   free (str2);
   
   return field_name;
@@ -671,24 +658,16 @@ rec_parse_record_str (char *str)
 {
   rec_parser_t parser;
   rec_record_t record;
-  FILE *stm;
-  size_t str_size;
 
   record = NULL;
-  str_size = strlen (str);
-  stm = fmemopen (str, str_size, "r");
-  if (stm)
+  parser = rec_parser_new_str (str, "dummy");
+  if (parser)
     {
-      parser = rec_parser_new (stm, "dummy");
-      if (parser)
+      if (!rec_parse_record (parser, &record))
         {
-          if (!rec_parse_record (parser, &record))
-            {
-              record = NULL;
-            }
-          rec_parser_destroy (parser);
+          record = NULL;
         }
-      fclose (stm);
+      rec_parser_destroy (parser);
     }
 
   return record;
@@ -703,7 +682,34 @@ rec_parser_getc (rec_parser_t parser)
 {
   int ci;
 
-  ci = getc (parser->in);
+  /* Get the input character depending on the backend used (memory or
+     file).  */
+  if (parser->in_file)
+    {
+      ci = getc (parser->in_file);
+    }
+  else if (parser->in_buffer)
+    {
+      if (*(parser->p) == '\0')
+        {
+          ci = EOF;
+        }
+      else
+        {
+          ci = *(parser->p);
+          parser->p++;
+        }
+    }
+  else
+    {
+      /* This point should not be reached!  */
+      fprintf (stderr, "rec_parser_getc: no backend in parser. This is a bug.\
+  Please report it.");
+      return EOF;
+    }
+
+  /* Manage EOF and update statistics.  */
+
   if (ci == EOF)
     {
       parser->eof = true;
@@ -726,16 +732,45 @@ rec_parser_ungetc (rec_parser_t parser,
 {
   int res;
 
+  /* Update statistics.  */
+
   parser->character--;
   if (((char) ci) == '\n')
     {
       parser->line--;
     }
 
-  res = ungetc (ci, parser->in);
-  if (res != ci)
+  /* Unread the character, depending on the backend used (memory or
+     file).  */
+
+  if (parser->in_file)
     {
-      parser->error = REC_PARSER_EUNGETC;
+      res = ungetc (ci, parser->in_file);
+      if (res != ci)
+        {
+          parser->error = REC_PARSER_EUNGETC;
+        }
+    }
+  else if (parser->in_buffer)
+    {
+      if (parser->p > parser->in_buffer)
+        {
+          res = ci; /* Emulate ungetc. */
+          parser->p--;
+        }
+      else
+        {
+          res = EOF;
+          parser->error = REC_PARSER_EUNGETC;
+        }
+    }
+  else
+    {
+
+      /* This point should not be reached!  */
+      fprintf (stderr, "rec_parser_ungetc: no backend in parser. This is a bug.\
+  Please report it.");
+      return EOF;
     }
 
   return res;
@@ -801,13 +836,12 @@ rec_parse_field_name_part (rec_parser_t parser,
 {
   bool ret;
   int ci;
-  size_t index;
+  size_t str_size;
   char c;
-  rec_parser_buf_t buf;
+  rec_buf_t buf;
 
   ret = true;
-  index = 0;
-  buf = rec_parser_buf_new ();
+  buf = rec_buf_new (str, &str_size);
   if (!buf)
     {
       /* Out of memory */
@@ -833,7 +867,7 @@ rec_parse_field_name_part (rec_parser_t parser,
       if ((rec_parser_letter_p (c))
           || (c == '%'))
         {
-          if (!rec_parser_buf_add (buf, c))
+          if (rec_buf_putc (c, buf) == EOF)
             {
               /* Out of memory */
               parser->error = REC_PARSER_ENOMEM;
@@ -860,7 +894,7 @@ rec_parse_field_name_part (rec_parser_t parser,
               || (c == '-')
               || (c == '_'))
             {
-              if (!rec_parser_buf_add (buf, c))
+              if (rec_buf_putc (c, buf) == EOF)
                 {
                   /* Out of memory */
                   parser->error = REC_PARSER_ENOMEM;
@@ -893,14 +927,12 @@ rec_parse_field_name_part (rec_parser_t parser,
         }
     }
 
-  if (ret)
-    {
-      /* Resize the token */
-      rec_parser_buf_adjust (buf);
-      *str = rec_parser_buf_data (buf);
-    }
+  rec_buf_close (buf);
 
-  rec_parser_buf_destroy (buf);
+  if (!ret)
+    {
+      free (*str);
+    }
 
   return ret;
 }
@@ -912,9 +944,9 @@ rec_parse_field_value (rec_parser_t parser,
   bool ret;
   int ci, ci2;
   char c, c2;
-  size_t index;
+  size_t str_size;
   bool prev_newline;
-  rec_parser_buf_t buf;
+  rec_buf_t buf;
 
   /* Sanity check */
   if (rec_parser_eof (parser)
@@ -926,8 +958,7 @@ rec_parse_field_value (rec_parser_t parser,
   c = '\0';
   prev_newline = false;
   ret = true;
-  index = 0;
-  buf = rec_parser_buf_new ();
+  buf = rec_buf_new (str, &str_size);
   if (!buf)
     {
       /* Out of memory */
@@ -949,7 +980,7 @@ rec_parse_field_value (rec_parser_t parser,
         {
           /* End of value */
           rec_parser_ungetc (parser, ci);
-          rec_parser_buf_rewind (buf, 1);
+          rec_buf_rewind (buf, 1);
           break;
         }
 
@@ -973,7 +1004,7 @@ rec_parse_field_value (rec_parser_t parser,
               else
                 {
                   /* Add \ and put back c2 */
-                  if (!rec_parser_buf_add (buf, c))
+                  if (rec_buf_putc (c, buf) == EOF)
                     {
                       /* Out of memory */
                       parser->error = REC_PARSER_ENOMEM;
@@ -1027,7 +1058,7 @@ rec_parse_field_value (rec_parser_t parser,
             }
           else
             {
-              if (!rec_parser_buf_add (buf, c))
+              if (rec_buf_putc (c, buf) == EOF)
                 {
                   /* Out of memory */
                   parser->error = REC_PARSER_ENOMEM;
@@ -1044,7 +1075,7 @@ rec_parse_field_value (rec_parser_t parser,
         }
       else if (c == '\n')
         {
-          if (!rec_parser_buf_add (buf, c))
+          if (rec_buf_putc (c, buf) == EOF)
             {
               /* Out of memory */
               parser->error = REC_PARSER_ENOMEM;
@@ -1059,7 +1090,7 @@ rec_parse_field_value (rec_parser_t parser,
         }
       else
         {
-          if (!rec_parser_buf_add (buf, c))
+          if (rec_buf_putc (c, buf) == EOF)
             {
               /* Out of memory */
               parser->error = REC_PARSER_ENOMEM;
@@ -1080,116 +1111,32 @@ rec_parse_field_value (rec_parser_t parser,
           && (c == '\n'))
         {
           /* Special case: field just before EOF */
-          rec_parser_buf_rewind (buf, 1);
-        }
-
-      /* Resize the token */
-     rec_parser_buf_adjust (buf);
-      *str = rec_parser_buf_data (buf);
-    }
-
-  rec_parser_buf_destroy (buf);
-
-  return ret;
-}
-
-static rec_parser_buf_t
-rec_parser_buf_new ()
-{
-  rec_parser_buf_t new;
-
-  new = malloc (sizeof (struct rec_parser_buf_s));
-  if (new)
-    {
-      new->data = malloc (REC_PARSER_BUF_STEP);
-      new->size = REC_PARSER_BUF_STEP;
-      new->used = 0;
-
-      if (!new->data)
-        {
-          free (new);
-          new = NULL;
+          rec_buf_rewind (buf, 1);
         }
     }
 
-  return new;
-}
+  rec_buf_close (buf);
 
-static void
-rec_parser_buf_destroy (rec_parser_buf_t buf)
-{
-  /* Don't deallocate buf->data */
-  free (buf);
-}
-
-static void
-rec_parser_buf_rewind (rec_parser_buf_t buf,
-                       int n)
-{
-  if ((buf->used - n) >= 0)
+  if (!ret)
     {
-      buf->used = buf->used - n;
-    }
-}
-
-static bool
-rec_parser_buf_add (rec_parser_buf_t buf,
-                    char c)
-{
-  bool ret;
-
-  ret = true;
-
-  if ((buf->used + 1) > buf->size)
-    {
-      /* Allocate a new block */
-      buf->size = buf->size + REC_PARSER_BUF_STEP;
-      buf->data = realloc (buf->data, buf->size);
-
-      if (!buf->data)
-        {
-          /* Not enough memory.
-           * REC_PARSER_BUF_STEP should not be 0. */
-          ret = false;
-        }
-    }
-
-  if (ret)
-    {
-      /* Add the character */
-      buf->data[buf->used++] = c;
+      free (*str);
     }
 
   return ret;
 }
 
-static char *
-rec_parser_buf_data (rec_parser_buf_t buf)
-{
-  return buf->data;
-}
-
-static void
-rec_parser_buf_adjust (rec_parser_buf_t buf)
-{
-  if (buf->used > 0)
-    {
-      buf->data = realloc (buf->data, buf->used + 1);
-    }
-
-  buf->data[buf->used] = '\0';
-}
-
 static bool
 rec_parse_comment (rec_parser_t parser, rec_comment_t *comment)
 {
   bool ret;
-  rec_parser_buf_t buf;
+  rec_buf_t buf;
+  char *str;
+  size_t str_size;
   int ci;
   char c;
 
   ret = false;
-  buf = rec_parser_buf_new ();
+  buf = rec_buf_new (&str, &str_size);
 
   /* Comments start at the beginning of line and span until the first
    * \n character not followed by a #, or EOF.
@@ -1219,7 +1166,7 @@ rec_parse_comment (rec_parser_t parser, rec_comment_t *comment)
                 }
             }
 
-          if (!rec_parser_buf_add (buf, c))
+          if (rec_buf_putc (c, buf) == EOF)
             {
               /* Out of memory */
               parser->error = REC_PARSER_ENOMEM;
@@ -1230,19 +1177,46 @@ rec_parse_comment (rec_parser_t parser, rec_comment_t *comment)
       ret = true;
     }
 
+  rec_buf_close (buf);
+
   if (ret)
     {
-      /* Resize the token */
-      rec_parser_buf_adjust (buf);
-      *comment = rec_comment_new (rec_parser_buf_data (buf));
+      *comment = rec_comment_new (str);
     }
   else
     {
+      free (str);
       *comment = NULL;
     }
 
-  rec_parser_buf_destroy (buf);
   return ret;
 }
 
+static bool
+rec_parser_init_common (rec_parser_t parser,
+                        char *source)
+{
+  if (source)
+    {
+      parser->source = strdup (source);
+      if (!parser->source)
+        {
+          return false;
+        }
+    }
+  else
+    {
+      parser->source = NULL;
+    }
+
+  parser->eof = false;
+  parser->error = false;
+  parser->line = 1;
+  parser->character = 0;
+  parser->prev_descriptor = NULL;
+  parser->p = parser->in_buffer;
+
+  return true;
+}
+
 /* End of rec-parser.c */
