@@ -24,11 +24,12 @@
  */
 
 #include <config.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <rec-mset.h>
 #include <rec.h>
+#include <rec-utils.h>
 
 /*
  * Record data structures.
@@ -522,10 +523,12 @@ rec_record_get_field_by_name (rec_record_t record,
   rec_mset_elem_t elem;
   int num_fields;
   rec_field_t field;
+  rec_field_t result;
 
   num_fields = 0;
   field = NULL;
   elem = NULL;
+  result = NULL;
 
   while ((elem = rec_mset_next (record->mset, elem, record->field_type)))
     {
@@ -534,6 +537,7 @@ rec_record_get_field_by_name (rec_record_t record,
         {
           if (n == num_fields)
             {
+              result = field;
               break;
             }
 
@@ -541,7 +545,7 @@ rec_record_get_field_by_name (rec_record_t record,
         }
     }
 
-  return field;
+  return result;
 }
 
 void
@@ -672,13 +676,13 @@ rec_record_elem_comment (rec_record_elem_t elem)
 rec_comment_t
 rec_record_to_comment (rec_record_t record)
 {
-  FILE *stm;
+  rec_buf_t buf;
   rec_comment_t res;
   char *comment_str;
   size_t comment_str_size;
   rec_record_elem_t elem;
 
-  stm = open_memstream (&comment_str, &comment_str_size);
+  buf = rec_buf_new (&comment_str, &comment_str_size);
 
   elem = rec_record_null_elem ();
   while (rec_record_elem_p (elem = rec_record_next (record, elem)))
@@ -686,20 +690,20 @@ rec_record_to_comment (rec_record_t record)
       if (rec_record_elem_field_p (record, elem))
         {
           /* Field.  */
-          fputs (rec_write_field_str (rec_record_elem_field (elem),
-                                      REC_WRITER_NORMAL),
-                 stm);
+          rec_buf_puts (rec_write_field_str (rec_record_elem_field (elem),
+                                             REC_WRITER_NORMAL),
+                        buf);
         }
       else
         {
           /* Comment.  */
-          fputs (rec_write_comment_str (rec_comment_text (rec_record_elem_comment (elem)),
-                                        REC_WRITER_NORMAL),
-                 stm);
+          rec_buf_puts (rec_write_comment_str (rec_comment_text (rec_record_elem_comment (elem)),
+                                               REC_WRITER_NORMAL),
+                        buf);
         }
     }
 
-  fclose (stm);
+  rec_buf_close (buf);
 
   /* Remove a trailing newline.  */
   if (comment_str[comment_str_size - 1] == '\n')
@@ -778,11 +782,7 @@ rec_record_set_location (rec_record_t record,
       record->location_str = NULL;
     }
 
-  record->location_str = malloc (30);
-  if (record->location_str)
-    {    
-      sprintf (record->location_str, "%d", record->location);
-    }
+  asprintf (&(record->location_str), "%zu", record->location);
 }
 
 size_t
@@ -820,11 +820,8 @@ rec_record_set_char_location (rec_record_t record,
       record->char_location_str = NULL;
     }
   
-  record->char_location_str = malloc (90);
-  if (record->char_location_str)
-    {
-      sprintf (record->char_location_str, "%d", record->char_location);
-    }
+  
+  asprintf (&(record->char_location_str), "%zu", record->char_location);
 }
 
 /*
