@@ -47,6 +47,28 @@
 void rec_init (void);
 void rec_fini (void);
 
+/*
+ * FLEXIBLE BUFFERS
+ *
+ * A flexible buffer (rec_buf_t) is a buffer to which stream-like
+ * operations can be applied.  Its size will grow as required.
+ */
+
+
+typedef struct rec_buf_s *rec_buf_t;
+
+rec_buf_t rec_buf_new (char **data, size_t *size);
+void rec_buf_close (rec_buf_t buffer);
+
+/* rec_buf_putc returns the character written as an unsigned char cast
+   to an int, or EOF on error.  */
+int rec_buf_putc (int c, rec_buf_t buffer);
+/* rec_buf_puts returns a non-negative number on success (number of
+   characters written), or EOF on error.  */
+int rec_buf_puts (const char *s, rec_buf_t buffer);
+
+void rec_buf_rewind (rec_buf_t buf, int n);
+
 /*
  * COMMENTS
  *
@@ -695,22 +717,22 @@ rec_rset_t rec_db_get_rset_by_type (rec_db_t db, const char *type);
 int rec_int_check_db (rec_db_t db,
                       bool check_descriptors_p,
                       bool remote_descriptors_p,
-                      FILE *errors);
+                      rec_buf_t errors);
 
 int rec_int_check_rset (rec_db_t db,
                         rec_rset_t rset,
                         bool check_descriptor_p,
                         bool remote_descriptor_p,
-                        FILE *errors);
+                        rec_buf_t errors);
 int rec_int_check_record (rec_db_t db,
                           rec_rset_t rset,
                           rec_record_t orig_rec,
                           rec_record_t rec,
-                          FILE *errors);
+                          rec_buf_t errors);
 bool rec_int_check_field_type (rec_db_t db,
                                rec_rset_t rset,
                                rec_field_t field,
-                               FILE *errors);
+                               rec_buf_t errors);
 
 /*
  * PARSER
@@ -721,9 +743,13 @@ bool rec_int_check_field_type (rec_db_t db,
 
 typedef struct rec_parser_s *rec_parser_t;
 
-/* Create a parser associated with a given file stream.  If not enough
-   memory, return NULL. */
-rec_parser_t rec_parser_new (FILE *in, char *file_name);
+/* Create a parser associated with a given file stream.
+   If not enough memory, return NULL. */
+rec_parser_t rec_parser_new (FILE *in, char *source);
+
+/* Create a parser associated with a given buffer.
+   If not enough memory, return NULL.  */
+rec_parser_t rec_parser_new_str (char *buffer, char *source);
 
 /* Destroy a parser.
  *
@@ -774,6 +800,10 @@ typedef struct rec_writer_s *rec_writer_t;
    memory, return NULL. */
 rec_writer_t rec_writer_new (FILE *out);
 
+/* Create a writer associated with a given string.  If not enough
+   memory, return NULL.  */
+rec_writer_t rec_writer_new_str (char **str, size_t *str_size);
+
 /* Destroy a writer.
  *
  * Note that this call is not closing the associated file stream.
@@ -832,6 +862,9 @@ bool rec_sex_compile (rec_sex_t sex, char *expr);
 /* Apply a sex expression to a record, setting RESULT in accordance.  */
 bool rec_sex_eval (rec_sex_t sex, rec_record_t record, bool *status);
 
+/* Apply a sex expression and get the result as a string.  */
+char *rec_sex_eval_str (rec_sex_t sex, rec_record_t record);
+
 void rec_sex_print_ast (rec_sex_t sex);
                              
 #endif /* !REC_H */
