@@ -29,7 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <xalloc.h>
-#include <libintl.h>
+#include <gettext.h>
 #define _(str) gettext (str)
 
 #include <rec.h>
@@ -43,7 +43,6 @@ void recdel_parse_args (int argc, char **argv);
  * Global variables
  */
 
-char *program_name; /* Initialized in main() */
 char *recutl_type = NULL;
 bool recdel_comment = false;
 rec_sex_t recutl_sex = NULL;
