@@ -29,7 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <xalloc.h>
-#include <libintl.h>
+#include <gettext.h>
 #define _(str) gettext (str)
 
 #include <rec.h>
@@ -39,7 +39,6 @@
  * Global variables
  */
 
-char *program_name; /* Initialized in main() */
 rec_writer_mode_t recinf_write_mode = REC_WRITER_NORMAL;
 
 /*
@@ -166,7 +165,10 @@ print_info_file (FILE *in,
                 }
               else
                 {
-                  printf ("unknown\n");
+                  if (recinf_write_mode == REC_WRITER_NORMAL)
+                    {
+                      printf ("unknown\n");
+                    }
                 }
 
               if (position < (rec_db_size (db) - 1))
