@@ -29,7 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <xalloc.h>
-#include <libintl.h>
+#include <gettext.h>
 #define _(str) gettext (str)
 
 #include <rec.h>
@@ -57,7 +57,6 @@ static void recset_process_del (rec_rset_t rset, rec_record_t record, rec_fex_t
 #define RECSET_ACT_COMMENT 4
 #define RECSET_ACT_RENAME  5
 
-char      *program_name       = NULL;
 char      *recutl_sex_str     = NULL;
 rec_sex_t  recutl_sex         = NULL;
 char      *recutl_fex_str     = NULL;
@@ -360,7 +359,7 @@ recset_process_actions (rec_db_t db)
   rec_record_t record;
   bool parse_status = true;
   rec_rset_elem_t rec_elem;
-  FILE *errors_stm;
+  rec_buf_t errors_buf;
   char *errors_str;
   size_t errors_str_size;
 
@@ -453,10 +452,10 @@ recset_process_actions (rec_db_t db)
       /* Check for integrity in the modified rset.  */
       if (!recset_force)
         {
-          errors_stm = open_memstream (&errors_str, &errors_str_size);
-          if (rec_int_check_rset (db, rset, false, recset_external, errors_stm) > 0)
+          errors_buf = rec_buf_new (&errors_str, &errors_str_size);
+          if (rec_int_check_rset (db, rset, false, recset_external, errors_buf) > 0)
             {
-              fclose (errors_stm);
+              rec_buf_close (errors_buf);
               if (!recset_verbose)
                 {
                   recutl_error (_("operation aborted due to integrity failures\n"));
